# bluetooth_device_row.py
#
# Copyright 2024 Christopher Talbot
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Adw
from gi.repository import Gtk
from gi.repository import Gio
from pubsub import pub
import gettext
import meshtastic.ble_interface

import gtk_meshtastic_client.utils as utils

@Gtk.Template(resource_path='/org/kop316/meshtastic/ui/bluetooth_device_row.ui')
class BluetoothDeviceRow(Adw.ActionRow):
    __gtype_name__ = 'BluetoothDeviceRow'

    connect_button = Gtk.Template.Child()

    def set_connection_page(self, connection_page):
        self.connection_page = connection_page

    def set_device_name(self, device_name):
        self.device_name = device_name
        self.set_title("Name: " + str(device_name))

    def set_device_address(self, device_address):
        self.device_address = device_address
        self.set_subtitle("Address: " + str(device_address))

    def _bluetooth_connect_finish(self, self_object, result: Gio.AsyncResult):
        app = Gtk.Application.get_default()
        win = Gtk.Application.get_active_window(app)

        self.bluetooth_scan_button.set_sensitive(True)
        self.populate_bluetooth_devices_list_box()

        if not Gio.Task.is_valid(result, self_object):
            print("Not valid GTask")
            win.connection_page_bin.set_disconnect_button_sensitivity(True)
            win.window_toast_overlay.add_toast(Adw.Toast(title="Bluetooth Connection Failed"))
            return

        success = result.propagate_boolean()
        if not success:
            print("Connection failed!")
            win.connection_page_bin.set_disconnect_button_sensitivity(True)
            win.window_toast_overlay.add_toast(Adw.Toast(title="Bluetooth Connection Failed"))
            return

    def _bluetooth_connect_async(self, callback) -> None:
        def _bluetooth_connect(task, _source_object, _task_data, _cancellable):
            print("Start Bluetooth Cpnnect")
            address_to_connect = self.device_address
            try:
                print("connect serial")
                print("Connecting to bluetooth interface: " + address_to_connect)
                meshtastic.ble_interface.BLEInterface(address_to_connect)
            except:
                task.return_boolean(False)
                return

            task.return_boolean(True)

        task = Gio.Task.new(self, None, callback)
        task.run_in_thread(_bluetooth_connect)

    @Gtk.Template.Callback()
    def _bluetooth_connect_button_clicked_cb(self, button):
        app = Gtk.Application.get_default()
        win = Gtk.Application.get_active_window(app)
        if not self.device_address:
            print("cannot connect, BLE Address not known")
            return

        if not hasattr(self, 'connection_page'):
            print("Cannot send message, connection page not defined!")
            return

        pub.subscribe(self.connection_page.onConnection, 'meshtastic.connection.established')
        self._bluetooth_connect_async(callback=self._bluetooth_connect_finish)
        win.connection_page_bin.set_disconnect_button_sensitivity(False)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
